package com.onespan.reportingtool.service;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.onespan.reportingtool.data.UserData;
import com.onespan.reportingtool.vo.AccountVo;
import com.onespan.reportingtool.vo.AuditEventVo;
import com.onespan.reportingtool.vo.PackageVo;

public class PackageService {
	private static PackageService endpointService;

	private PackageService() {
	}

	public static PackageService getInstance() {
		return endpointService == null ? new PackageService() : endpointService;
	}
	
	public void loadCompletedPackages(String from, String to) throws Exception {
		UserData.completedPackages.clear();
		
		// load packages for all senders
		Map<String, AccountVo> accountSenders = SenderService.getInstance().loadAllSendersByLoginVo(UserData.ownerCredential);
		for (Map.Entry<String, AccountVo> entry : accountSenders.entrySet()) {
			AccountVo accountSender = entry.getValue();
			System.out.println("getting packages for sender: " + accountSender.getEmail());
			loadPackagesForSender(from, to, accountSender);
		}
	}	
	
	private void loadPackagesForSender(String from, String to, AccountVo senderVo) throws MalformedURLException, JSONException {
		String query = "completed";
		int index = 0;
		int count = 0;
		int increment = 100;
		List<PackageVo> packages = new ArrayList<>();
		do {
			String jsonResponse = RestService.getInstance()
				.doGet(UserData.apiUrl + "/packages?from=" + (index + 1 ) + "&to=" + (index + increment)
					+ "&lastUpdatedStartDate=" + from + "&lastUpdatedEndDate=" + to
					+ "&query=" + query,
					senderVo);
			if (StringUtils.isNotEmpty(jsonResponse)) {
				JSONObject jsonObj = JSONObject.parseObject(jsonResponse);
	
				count = Integer.parseInt(jsonObj.getString("count"));
	
				String results = jsonObj.getString("results");
				List<PackageVo> listDTO = JSON.parseArray(results, PackageVo.class);
				
				for (PackageVo packageVo : listDTO) {
					// audit events for this package
					String auditJsonResponse = RestService.getInstance()
						.doGet(UserData.apiUrl + "/packages/" + packageVo.getId() + "/audit", senderVo);
					if (StringUtils.isNotEmpty(jsonResponse)) {
						JSONObject auditJsonObj = JSONObject.parseObject(auditJsonResponse);
						if (auditJsonObj != null) {
							List<AuditEventVo> auditevents = JSON.parseArray(auditJsonObj.getString("audit-events"), AuditEventVo.class);
							packageVo.setAuditEvents(auditevents);
						}
						
						packages.add(packageVo);
					}
				}
			}
			index += increment;
		} while (index <= count);
		if (!packages.isEmpty()) {
			// add to completed packages
			UserData.completedPackages.put(senderVo, packages);
		}
	}

}
