package com.onespan.reportingtool.service;

import static com.onespan.reportingtool.vo.AuditEventVo.TYPE.DOWNLOAD;
import static com.onespan.reportingtool.vo.AuditEventVo.TYPE.DOWNLOAD_ZIP;
import static com.onespan.reportingtool.vo.ReportVo.DOWNLOAD_STATUS.NO;
import static com.onespan.reportingtool.vo.ReportVo.DOWNLOAD_STATUS.YES;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;

import com.onespan.reportingtool.data.UserData;
import com.onespan.reportingtool.utils.FileUtil;
import com.onespan.reportingtool.view.DateLabelFormatter;
import com.onespan.reportingtool.vo.AccountVo;
import com.onespan.reportingtool.vo.AuditEventVo;
import com.onespan.reportingtool.vo.ChallengeVo;
import com.onespan.reportingtool.vo.DocumentVo;
import com.onespan.reportingtool.vo.PackageVo;
import com.onespan.reportingtool.vo.ReportColumnName;
import com.onespan.reportingtool.vo.ReportVo;
import com.onespan.reportingtool.vo.RoleVo;
import com.onespan.reportingtool.vo.SenderVo;
import com.onespan.reportingtool.vo.SignerVo;

import au.com.bytecode.opencsv.CSVWriter;

public class ReportService {
	private static ReportService reportService;
	private static final String[] columnNames = getColumnName();
	
	private ReportService() {
	}

	public static ReportService getInstance() {
		return reportService == null ? new ReportService() : reportService;
	}

	private static String[] getColumnName() {
		Field[] fields = ReportVo.class.getDeclaredFields();
		String[] values = new String[fields.length];
		int i = 0;
		for (Field field : fields) {
			String key = field.getAnnotation(ReportColumnName.class).value();
			try {
				values[i] = new String(UserData.mybundle.getString(key).getBytes("ISO-8859-1"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			i++;
		}
		return values;
	}
	
	private String[] getRowByReportVo(ReportVo reportVo) throws IllegalArgumentException, IllegalAccessException {
		Field[] fields = ReportVo.class.getDeclaredFields();
		String[] values = new String[fields.length];
		int i = 0;
		for (Field field : fields) {
			field.setAccessible(true);
		    values[i] = (String)field.get(reportVo);
		    i++;
		}
		return values;
	}
	
	/**
	 * @param columnName
	 * @param csvContent
	 * @return
	 * @throws IOException
	 */
    public byte[] resultSetAsCsvBytes(List<String[]> csvContent) throws IOException  {
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        OutputStreamWriter streamWriter = new OutputStreamWriter(stream, "UTF-8");
        
        CSVWriter writer = new CSVWriter(streamWriter, CSVWriter.DEFAULT_SEPARATOR, 
        		CSVWriter.NO_QUOTE_CHARACTER, CSVWriter.NO_ESCAPE_CHARACTER);
        writer.writeNext(ReportService.columnNames);
        writer.writeAll(csvContent);
        writer.close();
        return stream.toByteArray();
    }
    
    
    public String exportReport(String from, String to, boolean senderSpecificReport) throws IOException {
    	String outputPath = UserData.folderPath;
		List<String[]> csvContent = new ArrayList<>();
		for (Map.Entry<AccountVo, List<PackageVo>> entry : UserData.completedPackages.entrySet()) {
			if (senderSpecificReport) {
				csvContent = new ArrayList<>();
			}
			
			AccountVo sender = entry.getKey();
		    List<PackageVo> targetPackages = entry.getValue();
		   
		    // for each signer output separate row
	    	for(PackageVo packageVo : targetPackages) {
	    		for (RoleVo roleVo : packageVo.getRoles()) {
	    			if (!"SENDER".equals(roleVo.getType())) {
		    			for (SignerVo signerVo: roleVo.getSigners()) {
							ReportVo reportVO = populateReport(packageVo, signerVo);
							//System.out.println(reportVO);
							try {
								String[] rowByReportVo = getRowByReportVo(reportVO);
								csvContent.add(rowByReportVo);
								
							} catch (Exception e2) {
								e2.printStackTrace();
							}
		    			}
	    			}
	    		}
			}
	    	if (senderSpecificReport) {
	    		// output CSV to file
	    		writeReportFile(from, to,csvContent, sender);
	    	}
		}
		if (!senderSpecificReport) {
			outputPath = writeReportFile(from, to, csvContent, null);
		}
    	return outputPath;
    }
    
    private String writeReportFile(String from, String to, List<String[]> csvContent, AccountVo sender) throws IOException {
    	// limit to 10K per file
    	int rowLimit = 10000;
    	int size = csvContent.size();
		int pages = (int) Math.ceil((double)size / rowLimit);
	    
	    for (int currentPage = 1; currentPage <= pages; currentPage++) {
	    	List<String[]> subset = csvContent.stream()
	    			.collect(Collectors.toList())
	    			.subList((currentPage - 1) * rowLimit, Math.min(size, currentPage * rowLimit));
	    	byte[] report = resultSetAsCsvBytes(subset);
            
	    	DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyyMMdd'-'HHmmss");
	    	DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyyMMdd");
	    	DateTimeFormatter dtf3 = DateTimeFormatter.ofPattern(DateLabelFormatter.datePattern);
	    	
	    	StringBuffer buffer = new StringBuffer();
	    	buffer.append(UserData.ownerCredential.getId());
	    	buffer.append("_");
	    	buffer.append(dtf.format(LocalDateTime.now()));
	    	buffer.append("_");
	    	buffer.append(dtf2.format(dtf3.parse(from)));
	    	buffer.append("-");
	    	buffer.append(dtf2.format(dtf3.parse(to)));
	    	if (sender != null) {
	    		buffer.append("-");
	        	buffer.append(sender.getEmail().substring(0, sender.getEmail().lastIndexOf("@")));
	    	}
	    	buffer.append("-");
	    	buffer.append("Download_Status");
	    	buffer.append("_");
	    	buffer.append(currentPage);
	    	buffer.append("_");
	    	buffer.append(pages);
	    	buffer.append(".csv");
			String fileName = buffer.toString();
			fileName = FileUtil.validateFileName(fileName);
			String outputPath = UserData.folderPath + "//" + fileName;
	    	if(report != null) {
	    		FileUtil.writeByte(report, outputPath);
	    	}
	    }
	    return UserData.folderPath;
    }
    
    private ReportVo populateReport(PackageVo packageVo, SignerVo signerVo) {
    	ReportVo reportVo = new ReportVo();
		reportVo.setAccountId(UserData.ownerCredential.getId());
		reportVo.setAccountName(UserData.ownerCredential.getName());
		reportVo.setAccountEmail(UserData.ownerCredential.getEmail());
		reportVo.setTransactionID(packageVo.getId());
		reportVo.setTransactionURL("=HYPERLINK(\"" + UserData.baseUrl + "a/transaction/" + packageVo.getId() + "\")");
		reportVo.setCompletedDate(packageVo.getCompleted());
		reportVo.setLatestUpdatedDate(packageVo.getUpdated());
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
		
		Date retainUntil;
		try {
			if (packageVo.getRetainUntil() != null) {
				retainUntil = sdf.parse(packageVo.getRetainUntil());
	
				long diff = retainUntil.getTime() - (Instant.now().getEpochSecond() * 1000);
			    long days =  TimeUnit.DAYS.convert(diff, TimeUnit.MILLISECONDS);
			    reportVo.setDaysLeft(String.valueOf(days + 1));
			} else {
				reportVo.setDaysLeft("Unlimited");
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
		
		reportVo.setRetainUntil(packageVo.getRetainUntil());
		
		SenderVo sender = packageVo.getSender();
		
		reportVo.setSenderEmail(sender.getEmail());
		reportVo.setSenderName(sender.getFirstName() + " " + sender.getLastName());
		
		reportVo.setSignerEmail(signerVo.getEmail());
		reportVo.setSignerName(signerVo.getFirstName() + " " + signerVo.getLastName());
		if ("SMS".equals(signerVo.getAuth().getScheme())) {
			Optional<ChallengeVo> challenge = signerVo.getAuth().getChallenges().stream().findFirst();
			reportVo.setSignerMobile(challenge.isPresent() ? challenge.get().getQuestion() : "");
		}
		
		// get audit events just for this signer
		if (packageVo.getAuditEvents() != null && !packageVo.getAuditEvents().isEmpty()) {
			List<AuditEventVo> signerAuditEventList = packageVo.getAuditEvents().stream()
					 .filter(event -> event.getUserEmail().equals(signerVo.getEmail()) && 
							 (DOWNLOAD_ZIP.getLabel().equals(event.getType()) || DOWNLOAD.getLabel().equals(event.getType()))
							)
				        .collect(Collectors.toList());
		       
		    // check for download zip event
			boolean downloadedZip = signerAuditEventList.stream()
				.anyMatch(event -> DOWNLOAD_ZIP.getLabel().equals(event.getType()));
			
			if (downloadedZip) {
				reportVo.setDownloadStatus(YES.name());
			} else {
				boolean downloaded = true;
				List<String> notDownloadedDocList = new ArrayList<String>();
				// else check for individual download events and see which documents have been downloaded
				for (DocumentVo document : packageVo.getDocuments()) {
					boolean downloadedDoc = signerAuditEventList.stream()
							.anyMatch(event -> DOWNLOAD.getLabel().equals(event.getType()) && event.getTarget().equals(document.getName()));
					if (!downloadedDoc) {
						downloaded = false;
						notDownloadedDocList.add(document.getName());
					}
				}
				reportVo.setNotDownloadedDocumentList(StringUtils.join(notDownloadedDocList, '|'));
				reportVo.setDownloadStatus(downloaded ? YES.name() : NO.name());
			}
		}
		return reportVo;
    }
    
}
