package com.onespan.reportingtool.service;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.onespan.reportingtool.data.UserData;
import com.onespan.reportingtool.utils.HttpURLConnectionUtil;
import com.onespan.reportingtool.utils.SSLFix;
import com.onespan.reportingtool.utils.StringUtil;
import com.onespan.reportingtool.vo.AccountVo;

public class RestService {
	
	private final static Logger log = Logger.getLogger(RestService.class.getName());
	
	private static RestService endpointService;

	private RestService() {
	}

	public static RestService getInstance() {
		return endpointService == null ? new RestService() : endpointService;
	}


	public byte[] doGetByteArray(String url, AccountVo accountVo) throws Exception {
		SSLFix.execute();

		System.out.println("API Number: " + ++UserData.apiNum + "; Current time: " + new Date());
		
		url = StringUtil.removeSlashes(url);

		URL client = new URL(url);
		byte[] byteArray;
		
		HttpURLConnection conn;
		if(UserData.proxy != null) {
			conn = (HttpURLConnection) client.openConnection(UserData.proxy);
		}else {
			conn = (HttpURLConnection) client.openConnection();
		}
		
		HttpURLConnectionUtil.addCredential(conn, accountVo);
//		conn.setRequestProperty("Accept", "application/pdf; esl-api-version=11.21");
		
		InputStream inputStream = conn.getInputStream();
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();
		int nRead;
		byte[] data = new byte[1024];
		while ((nRead = inputStream.read(data, 0, data.length)) != -1) {
			buffer.write(data, 0, nRead);
		}

		buffer.flush();
		buffer.close();
		inputStream.close();
		conn.disconnect();

		byteArray = buffer.toByteArray();
		
		System.out.println(conn.getResponseCode());
		if(conn.getResponseCode() != 200) {
			throw new RuntimeException("Fail to download package: " + url);
		}
		
		return byteArray;
	}
	
	public String doGet(String url, AccountVo accountVo) throws MalformedURLException, JSONException {
		return doGet(url, accountVo, true, 0);
		
	}

	public String doGet(String url, AccountVo accountVo, boolean retry, Integer retryCount) throws MalformedURLException, JSONException {
		return execute(url, accountVo, null, null, true, retryCount);
	}
	
	public String doPost(String url, AccountVo accountVo, String payload) throws MalformedURLException, JSONException {
		return execute(url, accountVo, "POST", payload, true, 0);
	}
	
	public String doPost(String url, AccountVo accountVo, String payload, boolean retry, Integer retryCount) throws MalformedURLException, JSONException {
		return execute(url, accountVo, "POST", payload, retry, retryCount);
	}
	
	public String execute(String url, AccountVo accountVo, String method, String payload, boolean retry, Integer retryCount) throws MalformedURLException, JSONException {
		SSLFix.execute();
		String email = accountVo.getEmail();
		String sender = email != null ? " for sender " + email : "";
		System.out.println("API Number: " + ++UserData.apiNum + "; Current time: " + new Date() + sender);

		
		url = StringUtil.removeSlashes(url);
		URL client = new URL(url);
		HttpURLConnection conn;
		try {
			if (UserData.proxy != null) {
				conn = (HttpURLConnection) client.openConnection(UserData.proxy);
			} else {
				conn = (HttpURLConnection) client.openConnection();
			}
			
			conn.setRequestProperty("Content-Type", "application/json; esl-api-version=11.21");
			HttpURLConnectionUtil.addCredential(conn, accountVo);
			conn.setRequestProperty("Accept", "application/json; esl-api-version=11.21");
			if (method != null) {
				conn.setRequestMethod(method);
			}
			if (payload != null) {
				conn.setDoOutput(true);
				conn.setDoInput(true);
				
				OutputStream os = conn.getOutputStream();
				System.out.println(payload);
				os.write(payload.getBytes());
				os.flush();
				os.close();
			}
	
			int sourceResponseCode = ((HttpURLConnection) conn).getResponseCode();
			System.out.println("GET " + url + " : " +sourceResponseCode);
	
			if (sourceResponseCode == 200) {
				Reader ir = new InputStreamReader(conn.getInputStream(),"UTF-8");
				BufferedReader in = new BufferedReader(ir);
				String inputLine;
				StringBuffer response = new StringBuffer();
	
				while ((inputLine = in.readLine()) != null) {
					response.append(inputLine);
				}
	
				in.close();
				conn.disconnect();
	
				return response.toString();
			} else if (sourceResponseCode == 204) {	//no content
				return null;
			} else if (sourceResponseCode == 403) {	//no access (package has been deleted)
				return null;
			}
			else if (sourceResponseCode == 401 || sourceResponseCode == 302 || sourceResponseCode == 500 || sourceResponseCode == 407) {
				Reader ir = new InputStreamReader(conn.getErrorStream());
				BufferedReader in = new BufferedReader(ir);
				String inputLine;
				StringBuffer response = new StringBuffer();
	
				while ((inputLine = in.readLine()) != null) {
					response.append(inputLine);
				}
	
				in.close();
				conn.disconnect();
	
				if (response.toString().contains("accountSuspended")) {
					return null;
					
				} else if (retry && retryCount < 10) {
					retryCount++;
					// unauthorized token may have expired so get another one and try again once
					log.log(Level.INFO, "access token may have expired, requesting new one");
					System.out.println("access token may have expired, requesting new one");
		    		try {
						EndpointService.getInstance().getAccessToken();
					} catch (Exception e) {
						e.printStackTrace();
						return null;
					}
		    		return execute(url, UserData.ownerCredential, method, payload, true, retryCount);
				} else {
					throw new RuntimeException(response.toString());
				}
			} else {
				Reader ir = new InputStreamReader(conn.getErrorStream());
				BufferedReader in = new BufferedReader(ir);
				String inputLine;
				StringBuffer response = new StringBuffer();
	
				while ((inputLine = in.readLine()) != null) {
					response.append(inputLine);
				}
	
				in.close();
				conn.disconnect();
				throw new RuntimeException(response.toString());
			}
		} catch (IOException e) {
			System.out.println(e.getMessage());
			if (retry && retryCount < 10) {
				retryCount++;
				System.out.println("retrying request again");
				return execute(url, UserData.ownerCredential, method, payload, true, retryCount);
			}
			return "";
		}
	}

	
	public JSONArray doGetArray(String url, AccountVo accountVo) throws IOException, JSONException {
		SSLFix.execute();
		System.out.println("API Number: " + ++UserData.apiNum + "; Current time: " + new Date());

		
		url = StringUtil.removeSlashes(url);
		URL client = new URL(url);
		HttpURLConnection conn;
		if(UserData.proxy != null) {
			conn = (HttpURLConnection) client.openConnection(UserData.proxy);
		}else {
			conn = (HttpURLConnection) client.openConnection();
		}
		
		conn.setRequestProperty("Content-Type", "application/json; esl-api-version=11.21");
		HttpURLConnectionUtil.addCredential(conn, accountVo);
		conn.setRequestProperty("Accept", "application/json; esl-api-version=11.21");

		int sourceResponseCode = ((HttpURLConnection) conn).getResponseCode();
		System.out.println(url + " : " +sourceResponseCode);

		if (sourceResponseCode == 200) {
			Reader ir = new InputStreamReader(conn.getInputStream(),"UTF-8");
			BufferedReader in = new BufferedReader(ir);
			String inputLine;
			StringBuffer response = new StringBuffer();

			while ((inputLine = in.readLine()) != null) {
				response.append(inputLine);
			}

			in.close();
			conn.disconnect();

			return JSON.parseArray(response.toString());
		} else if (sourceResponseCode == 204) {	//no content
			return null;
		}else if (sourceResponseCode == 403) {	//no access (package has been deleted)
			return null;
		}
		else if (sourceResponseCode == 401) {
			Reader ir = new InputStreamReader(conn.getErrorStream());
			BufferedReader in = new BufferedReader(ir);
			String inputLine;
			StringBuffer response = new StringBuffer();

			while ((inputLine = in.readLine()) != null) {
				response.append(inputLine);
			}

			in.close();
			conn.disconnect();

			if (response.toString().contains("accountSuspended")) {
				return null;
			} else {
				throw new RuntimeException(response.toString());
			}
		} else {
			Reader ir = new InputStreamReader(conn.getErrorStream());
			BufferedReader in = new BufferedReader(ir);
			String inputLine;
			StringBuffer response = new StringBuffer();

			while ((inputLine = in.readLine()) != null) {
				response.append(inputLine);
			}

			in.close();
			conn.disconnect();
			throw new RuntimeException(response.toString());
		}
	}


}
