package com.onespan.reportingtool.service;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.onespan.reportingtool.data.UserData;
import com.onespan.reportingtool.vo.AccountVo;
import com.onespan.reportingtool.vo.AccountVo.CredentialType;

public class SenderService {
	private static SenderService senderService;

	private SenderService() {
	}

	public static SenderService getInstance() {
		return senderService == null ? new SenderService() : senderService;
	}

	public Map<String, AccountVo> loadAllSendersByLoginVo(AccountVo loginUserVo) throws MalformedURLException, JSONException {
		Map<String, AccountVo> senderVos = new HashMap<>();
		int index = 0;
	    int count = 0;
	    int increment = 100;
		do {
			String jsonResponse = RestService.getInstance()
					.doGet(UserData.apiUrl + "/account/senders?from=" + (index + 1) + "&to=" + (index + increment), loginUserVo);
			JSONObject senders = JSON.parseObject(jsonResponse);
			if (StringUtils.isNotEmpty(jsonResponse)) {
				JSONArray results = senders.getJSONArray("results");
				count =  Integer.parseInt(senders.getString("count"));
	
				for (int i = 0; i < results.size(); i++) {
					JSONObject curSender = results.getJSONObject(i);
	
					//step0: make sure sender is not suspended
					if (curSender.getString("status").equals("LOCKED") || curSender.getString("status").equals("INVITED")) {
						continue;
					}
	
					String senderId = curSender.getString("id");
	
					try {
						//step1: retrieve sender API Key
						//String senderApiKeyResponse = RestService.getInstance()
						//		.doGet(UserData.apiUrl + "/account/senders/" + senderId + "/apiKey", loginUserVo);
						//JSONObject senderApiKeyObj = JSON.parseObject(senderApiKeyResponse);
						//String apikey = senderApiKeyObj.getString("apiKey");
						
						String decoded = new String(Base64.getDecoder().decode(loginUserVo.getApiKey()));
						String senderDecoded = senderId + decoded.substring(decoded.indexOf(':'));
						String apikey = Base64.getEncoder().encodeToString(senderDecoded.getBytes());

						AccountVo senderVo = new AccountVo();
						senderVo.setApiKey(apikey);
						senderVo.setCredential(apikey);
						senderVo.setCredentialType(CredentialType.API_KEY);
	
						senderVo.setId(senderId);
						senderVo.setFirstName(curSender.getString("firstName"));
						senderVo.setLastName(curSender.getString("lastName"));
						senderVo.setEmail(curSender.getString("email"));
	
						senderVos.put(senderId, senderVo);
					} catch (Exception e) {
						// if account owner is not the physical owner of this sender, API returns 404
						// when querying sender api key
						e.printStackTrace();
					}
				}
			}
			index += increment;
		} while (index <= count);

		return senderVos;
	}

	public AccountVo injectAccountVo(String url, AccountVo accountVo) throws IOException, JSONException {
		/*
		String payload = "{\r\n" + "	\"clientId\":\"" + accountVo.getAppId() + "\",\r\n" + "	\"secret\":\""
				+ accountVo.getAppSecret() + "\",\r\n" + " \"type\":\"OWNER\"\r\n }";
		
		String r = RestService.getInstance().doPost(url, accountVo, payload, false, 0);
			
		if (r != null) {
			TokenVo token = JSON.parseObject(r, TokenVo.class);			
			accountVo.setCredential(token.getAccessToken());
		} else {
			throw new RuntimeException("Credentials for app id " + accountVo.getAppId()
					+ " is not correct, please check again! Reason: " );
		}
		*/
		
		try {
			String payload = String.format("{\"clientId\":\"%1$s\",\"secret\":\"%2$s\",\"type\":\"OWNER\"}", 
					accountVo.getAppId(),accountVo.getAppSecret());
			String postStr = RestService.getInstance().doPost(UserData.baseUrl + 
					"/apitoken/clientApp/accessToken", accountVo, payload);
			JSONObject doPost = JSON.parseObject(postStr);
			String accessToken = doPost.getString("accessToken");
			accountVo.setCredential(accessToken);
			
			String getStr = RestService.getInstance().doGet(UserData.apiUrl + "/session/apiKey", accountVo);
			JSONObject doGet = JSON.parseObject(getStr);
			String apiKey = doGet.getString("apiKey");
			accountVo.setApiKey(apiKey);
			accountVo.setCredential(apiKey);
			accountVo.setCredentialType(CredentialType.API_KEY);
		} catch(Exception e) {
			throw new RuntimeException("Credentials for app id " + accountVo.getAppId()
			+ " is not correct, please check again! " );
		}
		return accountVo;
	}

	public void loadAccountInfo() throws Exception {
		try {
			// load account info
			String jsonResponse = RestService.getInstance()
					.doGet(UserData.apiUrl + "/account", UserData.ownerCredential);
			if (StringUtils.isNotEmpty(jsonResponse)) {
				JSONObject jsonObj = JSONObject.parseObject(jsonResponse);
				String accountId = jsonObj.getString("id");
				String ownerId = jsonObj.getString("owner");
				UserData.ownerCredential.setId(accountId);
						
				String senderJsonResponse = RestService.getInstance()
						.doGet(UserData.apiUrl + "/account/senders/" + ownerId, UserData.ownerCredential);
	
				if (StringUtils.isNotEmpty(jsonResponse)) {
					JSONObject senderJsonObj = JSONObject.parseObject(senderJsonResponse);
					UserData.ownerCredential.setFirstName(senderJsonObj.getString("firstName"));
					UserData.ownerCredential.setLastName(senderJsonObj.getString("lastName"));
					UserData.ownerCredential.setEmail(senderJsonObj.getString("email"));
				}
			}
		} catch (Exception e) {
			throw e;
		}
	}
	
	

}
